﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.DbExplorer.Interfaces;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.DataProcessor.Command {
  [StorableClass]
  public class LoadColumnGroupWithSqlStringFromDBCommand : DatabaseCommandBase {
    [Storable]
    private string sqlString;

    private ColumnGroup newColumnGroup;

    private LoadColumnGroupWithSqlStringFromDBCommand()
      : base(null, null) {
    }

    public LoadColumnGroupWithSqlStringFromDBCommand(DataSet dataSet, IDbExplorer dbExplorer, string sqlString)
      : base(dataSet, dbExplorer) {
      this.sqlString = sqlString;
    }

    public override string Description {
      get { return "Load ColumnGroups from database"; }
    }

    public override void Execute() {
      base.Execute();
      if (DbExplorer == null || !DbExplorer.IsConnected)
        throw new CommandExecutionException("Could not load data because no connection to the database exists", this);
      newColumnGroup = DbExplorer.CreateColumnGroup(sqlString);
      newColumnGroup.Name = "New ColumnGroup from SQL string";
      this.DataSet.AddColumnGroup(newColumnGroup);
      this.DataSet.FireChanged();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      this.DataSet.RemoveColumnGroup(newColumnGroup);
      newColumnGroup = null;
      this.DataSet.FireChanged();
    }
  }
}
