﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.DataProcessor.Command {
  [StorableClass]
  public class ReorderColumnsCommand : DataSetCommandBase {
    [Storable]
    private Dictionary<string, int[]> displayedIndexes;

    private ReorderColumnsCommand()
      : base(null) {
    }

    public ReorderColumnsCommand(DataSet ds, Dictionary<string, int[]> displayedIndexes)
      : base(ds) {
      this.displayedIndexes = displayedIndexes;
    }

    public override string Description {
      get { return "Reorder Columns"; }
    }

    public override void Execute() {
      base.Execute();
      ReorderColumns(true);
      this.DataSet.FireChanged();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      ReorderColumns(false);
      this.DataSet.FireChanged();
    }

    private void ReorderColumns(bool forward) {
      ColumnGroup grp;
      int[] indices;
      List<int> sortedColumnIndices;
      int newIndex;
      int oldIndex;
      List<ColumnBase> columns;
      foreach (KeyValuePair<string, int[]> pair in displayedIndexes) {
        grp = this.DataSet.GetColumnGroup(pair.Key);
        indices = pair.Value;
        columns = grp.Columns.ToList();
        sortedColumnIndices = new List<int>(grp.SortedColumnIndexes);
        for (int j = 0; j < indices.Length; j++) {
          sortedColumnIndices = new List<int>(grp.SortedColumnIndexes);
          if (forward) {
            newIndex = j;
            oldIndex = indices[j];
          } else {
            newIndex = indices[j];
            oldIndex = j;
          }
          grp.ReplaceColumn(newIndex, columns[oldIndex]);
          grp.SortedColumnIndexes = sortedColumnIndices;
          //corrects the SortedColumnIndices if sorted colum got reordered
          if (grp.SortedColumnIndexes.Contains(oldIndex)) {
            int insertPos = sortedColumnIndices.IndexOf(oldIndex);
            sortedColumnIndices.Remove(oldIndex);
            sortedColumnIndices.Insert(insertPos, newIndex);
          }
        }
        grp.SortedColumnIndexes = sortedColumnIndices;
        grp.FireChanged();
      }
    }
  }
}
